unit Unit_LGS;

interface

const MaxCount = 10;  // maximale Zahl der Gleichungen und Variablen

type TEquation = Array [0..MaxCount] of Double;
     { enthlt die  Koeffizienten einer linearen Gleichung;
       an der i-ten Stelle steht der Koeffizient der Variablen xi,
       wobei i von 1 bis VarCount luft;
       an der nullten Stelle steht der Wert der "rechten Seite". }

var ko : Array [0..MaxCount, 0..MaxCount] of Double;  { Details siehe unten }
    EquaCount,
    VarCount : Integer;

procedure SetEquation(nr: Integer; koeff: TEquation);
  { schreibt die Koeffizienten der bergebenen Gleichung koeff
    als nr-te Gleichung in das Gleichungssystem }

procedure GetEquation(nr: Integer; var koeff: TEquation);
  { liest die Koeffizienten der nr-ten Gleichung aus dem Gleichungssystem
    und bergibt sie in der Gleichung koeff }

procedure ExchangeEquations(n1, n2: Integer);
  { vertauscht die n1-te mit der n2-ten Gleichung }

procedure DivideEquation(nr: Integer; factor: Double);
  { diviert die nr-te Gleichung durch factor }

procedure SubtEquation(dest: Integer; factor: Double; source: Integer);
  { subtrahiert von der dest-ten Gleichung das factor-fache der
    source-ten Gleichung und schreibt die neuen Koeffizienten in die
    dest-te Gleichung zurck }

procedure Manipulate;
  { Spielwiese fr Tests }



implementation

{ Fr die Interpretation der Indizes von ko gilt im folgenden :
      ko [Variablennummer, Gleichungsnummer]
  in Analogie zu: "zuerst" die "x"-Koordinate, dann die "y"-Koordinate.

  Ein StringGrid ist brigens genau so organisiert:
      Cells [Spalte, Zeile]

  Allerdings geben wir die Konstanten der rechten Seite unseres LGS im
  StringGrid in einer zustzlichen rechten Spalte an, also z.B.
     bei einem 3x3-System in der Spalte mit der Nummer 4, die aber
     eigentlich die 5. Spalte ist!

  Im Koeffizienten-Array sollen diese Konstanten nun in der Spalte mit
  der Nummer 0 abgelegt werden. Das muss schon das aufrufende Programm
  bercksichtigen, wenn es die Koeffizienten fr die Transfer-Prozeduren
  SetEquation() und GetEquation() in die TEquation-Variablen koeff packt
  bzw. aus ihr ausliest!                                                 }


procedure SetEquation(nr: Integer; koeff: TEquation);
  var i : Integer;
  begin
  For i := 0 to Varcount do
    ko[i, nr] := koeff[i];
  end;

procedure GetEquation(nr: Integer; var koeff: TEquation);
  var i : Integer;
  begin
  For i := 0 to VarCount do
    koeff[i] := ko[i, nr];
  end;

procedure ExchangeEquations(n1, n2: Integer);
  var pu : Double;
      i  : Integer;
  begin
  If n1 <> n2 then
    For i := 0 to VarCount do begin
      pu        := ko[i, n1];
      ko[i, n1] := ko[i, n2];
      ko[i, n2] := pu;
      end;
  end;

procedure DivideEquation(nr: Integer; factor: Double);
  var i : Integer;
  begin
  For i := 0 to VarCount do
    ko[i, nr] := ko[i, nr] / factor;
  end;

procedure SubtEquation(dest: Integer; factor: Double; source: Integer);
  var i : Integer;
  begin
  For i := 0 to VarCount do
    ko[i, dest] := ko[i, dest] - ko[i, source] * factor;
  end;

procedure Manipulate;
  { Hier ist ein sehr einfacher Gau-Algorithmus zur vollstndigen
          Diagonalisierung eines 3x3-Systems "per Hand" . Er kommt
          noch ohne "ExchangeEquations" aus.
          Im Augenblick ist nur die Bearbeitung der ersten Zeile aktiv. }
  begin
  DivideEquation(1, ko[1, 1]);      // normieren
  SubtEquation  (2, ko[1, 2], 1);   //   leerputzen
  SubtEquation  (3, ko[1, 3], 1);   //   leerputzen
{
  DivideEquation(2, ko[2, 2]);      // normieren
  SubtEquation  (1, ko[2, 1], 2);   //   leerputzen
  SubtEquation  (3, ko[2, 3], 2);   //   leerputzen

  DivideEquation(3, ko[3, 3]);      // normieren
  SubtEquation  (1, ko[3, 1], 3);   //   leerputzen
  SubtEquation  (2, ko[3, 2], 3);   //   leerputzen
}
  end;

end.
